SystemOrganization addCategory: #GitFS!
SystemOrganization addCategory: #'GitFS-Tests'!

FSFilesystem subclass: #GitFS
	instanceVariableNames: 'repository workingCopy modified objectMap deletions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS'!

!GitFS class methodsFor: 'instance creation' stamp: 'MaxLeske 7/8/2010 12:32'!
for: aGitRepository
	^self basicNew initializeOn: aGitRepository! !

!GitFS class methodsFor: 'instance creation' stamp: 'MaxLeske 7/8/2010 12:29'!
new
	self error: 'Please use the constructor methods to create instances.'! !

!GitFS methodsFor: 'private' stamp: 'MaxLeske 7/9/2010 12:08'!
basicIsDirectory: anEntry
	^ workingCopy basicIsDirectory: anEntry! !

!GitFS methodsFor: 'private' stamp: 'MaxLeske 7/9/2010 12:11'!
basicIsFile: entry
	^ workingCopy basicIsFile: entry! !

!GitFS methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 11:56'!
changeDirectory: aPath
	workingCopy changeDirectory: aPath! !

!GitFS methodsFor: 'public' stamp: 'MaxLeske 7/9/2010 11:50'!
childrenAt: anObject
	[ ^ workingCopy childrenAt: anObject ]
		on: FSDirectoryDoesNotExist 
		do: [ ^ modified childrenAt: anObject ]! !

!GitFS methodsFor: 'git' stamp: 'MaxLeske 7/8/2010 16:25'!
commit: aString
	self notYetImplemented! !

!GitFS methodsFor: 'initialize-release' stamp: 'MaxLeske 7/9/2010 15:21'!
copyToMemory: aGitTreeEntry at: aPath
	| entryPath entryObject |
	entryPath := aPath / (aGitTreeEntry fileName).
	entryObject := repository objectWithSignature: aGitTreeEntry objectSignature.
	objectMap at: entryPath put: entryObject.
	
	aGitTreeEntry mode = (GitTreeEntry dirMode)
		ifTrue: [
			workingCopy createDirectory: entryPath.
			entryObject entries do: [ :entry | self copyToMemory: entry at: entryPath ] ]
		ifFalse: [
			workingCopy createFile: entryPath.
			(workingCopy root resolve: entryPath)
				writeStreamDo:	[ :stream | stream nextPutAll: entryObject bytes ] ]! !

!GitFS methodsFor: 'public' stamp: 'MaxLeske 7/8/2010 11:54'!
createDirectory: aPath
	modified createDirectory: aPath! !

!GitFS methodsFor: 'public' stamp: 'MaxLeske 7/9/2010 11:46'!
delete: aPath
	| fileNotFound |
	fileNotFound := false.
	workingCopy
		nodeAt: aPath
		ifPresent: [ :node |
			workingCopy delete: aPath.
			deletions add: aPath]
		ifAbsent: [ fileNotFound := true ].
	modified
		nodeAt: aPath
		ifPresent: [ :node | 
			modified delete: aPath.
			fileNotFound := false ]
		ifAbsent: [ "do nothing" ].
		
	fileNotFound ifTrue: [ self fileDoesNotExist: aPath ]! !

!GitFS methodsFor: 'public' stamp: 'MaxLeske 7/8/2010 10:58'!
delimiter
	^ $/! !

!GitFS methodsFor: 'private' stamp: 'MaxLeske 7/9/2010 09:40'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	modified 
		directoryAt: aPath 
		ifAbsent: [ 
			workingCopy 
				directoryAt: aBlock
			 	ifAbsent: absentBlock 
				nodesDo: aBlock]
		nodesDo: aBlock! !

!GitFS methodsFor: 'public' stamp: 'MaxLeske 7/9/2010 12:12'!
entriesAt: anObject
	[ ^ workingCopy entriesAt: anObject ]
		on: FSDirectoryDoesNotExist 
		do: [ ^ modified entriesAt: anObject ]! !

!GitFS methodsFor: 'private' stamp: 'MaxLeske 7/9/2010 12:17'!
entryFromNode: node atPath: aPath

	^ FSDirectoryEntry
		filesystem: self
		path: aPath
		creation: 0
		modification: 0
		isDir: node isDictionary
		size: (node isDictionary ifTrue: [0] ifFalse: [node size])! !

!GitFS methodsFor: 'printing' stamp: 'MaxLeske 7/8/2010 11:06'!
forReferencePrintOn: aStream 
	aStream nextPutAll: 'git:'! !

!GitFS methodsFor: 'initialize-release' stamp: 'MaxLeske 7/9/2010 15:17'!
initWorkingCopy
	| head |
	head := repository master.
	head ifNil: [ ^ self ].
	
	head tree entries do: [ :entry | 
		self copyToMemory: entry at: FSPath new ]! !

!GitFS methodsFor: 'initialize-release' stamp: 'MaxLeske 7/9/2010 11:46'!
initializeOn: aGitRepository
	super initialize.
	workingCopy := FSMemoryFilesystem new.
	modified := FSMemoryFilesystem new.
	objectMap := Dictionary new.
	deletions := Set new.
	repository := aGitRepository.
	self initWorkingCopy! !

!GitFS methodsFor: 'internal' stamp: 'MaxLeske 7/9/2010 11:49'!
moveToModified: aPath
	(workingCopy root resolve: aPath) copyTo: (modified root resolve: aPath)! !

!GitFS methodsFor: 'private' stamp: 'MaxLeske 7/9/2010 12:47'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	^ modified 
		nodeAt: aPath 
		ifPresent: presentBlock
		ifAbsent: [ 
			workingCopy 
				nodeAt: aPath
				ifPresent: presentBlock
			 	ifAbsent: absentBlock]! !

!GitFS methodsFor: 'public' stamp: 'MaxLeske 7/9/2010 12:01'!
open: anObject writable: aBoolean
	| path |
	path := self resolve: anObject.
	modified
		nodeAt: path
		ifPresent: [ :node | ^ modified open: path writable: aBoolean ]
		ifAbsent: [
			aBoolean
				ifTrue: [ 
					workingCopy
						nodeAt: path
						ifPresent: [ :node | 
							self copyToModified: path.
							self open: anObject writable: aBoolean ]
						ifAbsent: [ ^ modified open: path writable: aBoolean ] ]
				ifFalse: [
					workingCopy
						open: path
						writable: aBoolean ] ]! !

!GitFS methodsFor: 'git' stamp: 'MaxLeske 7/8/2010 12:13'!
reset
	modified := FSMemoryFilesystem new! !

!GitFS methodsFor: 'public' stamp: 'MaxLeske 7/9/2010 11:39'!
resolve: anObject
	| workingFSPath modifiedFSPath |
	workingFSPath := anObject asResolvedBy: workingCopy.
	modifiedFSPath := anObject asResolvedBy: modified.
	
	modified
		nodeAt: modifiedFSPath
		ifPresent: [ :node | ^ modifiedFSPath ]
		ifAbsent: [ ^ workingFSPath ]! !

!GitFS methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 11:51'!
root
	^ workingCopy root! !

!GitFS methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 11:51'!
working
	^ workingCopy working! !

!GitFS methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 11:58'!
workingDirectory
	^ workingCopy workingDirectory! !

!GitFS methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 11:55'!
workingDirectory: aPath
	workingCopy workingDirectory: aPath! !

FSFilesystemTest subclass: #GitFSTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS-Tests'!

!GitFSTests class methodsFor: 'package selection' stamp: 'MaxLeske 7/9/2010 10:29'!
packageNamesUnderTest
	^ #('GitFS')! !

!GitFSTests methodsFor: 'initialize-release' stamp: 'MaxLeske 7/8/2010 16:55'!
createFilesystem
	 | repo |
	repo := GitRepository on: (FSMemoryFilesystem new root resolve: 'testRepo.git').
	repo gitInitialize.
	^GitFS for: repo! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 7/9/2010 11:53'!
testRoot
	self assert: filesystem root filesystem species = FSMemoryFilesystem.
	self assert: filesystem root path = FSPath root! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 7/9/2010 11:52'!
testWorking
	self assert: filesystem working filesystem species = FSMemoryFilesystem.
	self assert: filesystem working path = filesystem workingDirectory! !

Object subclass: #GitFSRepository
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS'!

!GitFSRepository class methodsFor: 'as yet unclassified' stamp: 'MaxLeske 7/9/2010 15:10'!
on: anFSReference
	^ self new initializeOn: anFSReference! !

!GitFSRepository methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 15:04'!
branches
	self notYetImplemented ! !

!GitFSRepository methodsFor: 'initialize-release' stamp: 'MaxLeske 7/9/2010 15:12'!
initializeOn: anFSReference
	repository := GitRepository on: anFSReference! !

!GitFSRepository methodsFor: 'accessing' stamp: 'MaxLeske 7/9/2010 15:16'!
master
	^ GitFS for: repository! !

!GitFSRepository methodsFor: 'as yet unclassified' stamp: 'MaxLeske 7/9/2010 15:06'!
tags
	self notYetImplemented ! !

!GitFSRepository methodsFor: 'as yet unclassified' stamp: 'MaxLeske 7/9/2010 15:07'!
versions
	self notYetImplemented ! !
